"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/// <reference types="@types/jest" />
// eslint-disable-next-line
const react_1 = tslib_1.__importDefault(require("react"));
const react_test_renderer_1 = tslib_1.__importDefault(require("react-test-renderer"));
const execaMock = jest.fn();
jest.mock('execa', () => ({
    __esModule: true,
    default: execaMock,
}));
const spinnerFailMock = jest.fn();
jest.mock('@dash4/log', () => ({
    __esModule: true,
    spinner: () => {
        return {
            start: jest.fn(),
            succeed: jest.fn(),
            fail: spinnerFailMock,
            text: jest.fn(),
        };
    },
}));
jest.mock('chalk', () => ({
    __esModule: true,
    default: {
        // eslint-disable-next-line
        hex: (vaui) => (text) => text,
        bgBlack: (text) => text,
        white: (text) => text,
        red: (text) => text,
        bold: (text) => text,
    },
}));
const cpy_1 = tslib_1.__importDefault(require("cpy"));
const del_1 = tslib_1.__importDefault(require("del"));
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const make_dir_1 = tslib_1.__importDefault(require("make-dir"));
const path_1 = tslib_1.__importDefault(require("path"));
const read_pkg_1 = tslib_1.__importDefault(require("read-pkg"));
const _1 = require(".");
const utils_1 = require("./utils");
const tempDir = path_1.default.join(__dirname, '../__tmp__');
const getTempFile = (testId, fileName) => tslib_1.__awaiter(void 0, void 0, void 0, function* () { return yield fs_extra_1.default.readFile(path_1.default.join(tempDir, testId, fileName), 'utf8'); });
const mockDir = path_1.default.join(__dirname, '../src/__mocks__');
const processKill = process.kill;
// eslint-disable-next-line
const consoleLog = console.log;
process.kill = jest.fn();
// eslint-disable-next-line
console.log = jest.fn();
const getOptions = (cwd, force = true) => ({
    port: 4000,
    config: path_1.default.join(cwd, 'dash4.config.js'),
    force,
});
function runSnapshotTest(testId, dependencies) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const cwd = path_1.default.posix.join(tempDir, testId);
        yield make_dir_1.default(cwd);
        yield cpy_1.default([`src/__mocks__/${testId}/_package.json`], `__tmp__/${testId}`, {
            rename: () => `package.json`,
        });
        if (yield utils_1.hasFile(mockDir, testId, `README.md`)) {
            yield cpy_1.default([`src/__mocks__/${testId}/README.md`], `__tmp__/${testId}`);
        }
        if (yield utils_1.hasFile(mockDir, testId, `readme.md`)) {
            yield cpy_1.default([`src/__mocks__/${testId}/readme.md`], `__tmp__/${testId}`);
        }
        yield _1.init(cwd, getOptions(cwd));
        expect(execaMock).toHaveBeenCalledWith('npm', `i -D @dash4/server ${dependencies}`.split(' '));
        expect(((yield read_pkg_1.default({ cwd })) || {}).scripts.dash4).toBe('dash4');
        const fileData = yield getTempFile(testId, 'dash4.config.js');
        const tree = react_test_renderer_1.default.create(react_1.default.createElement("div", { dangerouslySetInnerHTML: { __html: fileData } })).toJSON();
        expect(tree).toMatchSnapshot();
    });
}
describe('cli', () => {
    beforeEach(() => {
        jest.resetAllMocks();
    });
    beforeAll(() => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield del_1.default(tempDir);
        // create needed directories and files
        yield make_dir_1.default(tempDir);
    }));
    afterAll(() => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        // eslint-disable-next-line
        console.log = consoleLog;
        process.kill = processKill;
        // await del(tempDir);
    }));
    test('is function', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        expect(typeof _1.init).toBe('function');
    }));
    test('execute in empty directory should fail with error', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const cwd = path_1.default.join(tempDir, 'empty');
        yield make_dir_1.default(cwd);
        const spyProcess = jest.spyOn(process, 'kill');
        yield _1.init(cwd, getOptions(cwd));
        expect(spinnerFailMock).toHaveBeenCalledWith('package.json not found!');
        expect(spyProcess).toHaveBeenCalledWith(1);
    }));
    test('execute in directory where config exist', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const cwd = path_1.default.join(tempDir, 'config-exist');
        yield make_dir_1.default(cwd);
        yield fs_extra_1.default.writeFile(path_1.default.join(cwd, 'package.json'), '{ "scripts": { "start": "dash4" } }');
        yield fs_extra_1.default.writeFile(path_1.default.join(cwd, 'dash4.config.js'), '');
        const spyProcess = jest.spyOn(process, 'kill');
        yield _1.init(cwd, getOptions(cwd, false));
        expect(spinnerFailMock).toHaveBeenCalledWith('Dash4 is already installed!');
        expect(spyProcess).toHaveBeenCalledWith(1);
    }));
    test('execute in directory where config exist but with force mode', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const cwd = path_1.default.join(tempDir, 'config-exist');
        yield make_dir_1.default(cwd);
        yield fs_extra_1.default.writeFile(path_1.default.join(cwd, 'package.json'), '{ "scripts": { "start": "dash4" } }');
        yield fs_extra_1.default.writeFile(path_1.default.join(cwd, 'dash4.config.js'), '');
        yield _1.init(cwd, getOptions(cwd, true));
        expect(((yield read_pkg_1.default({ cwd })) || {}).scripts.dash4).toBe('dash4');
    }));
    test('execute with script "start"', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield runSnapshotTest('script-start', '@dash4/plugin-dependencies @dash4/plugin-terminal');
    }));
    test('execute with "README.md" file', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        expect(yield utils_1.hasFile(mockDir, 'readme', `README.md`)).toBe(true);
        yield runSnapshotTest('readme', '@dash4/plugin-dependencies @dash4/plugin-readme');
    }));
    test('execute with script "test"', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield runSnapshotTest('script-test', '@dash4/plugin-dependencies @dash4/plugin-code-coverage @dash4/plugin-npm-scripts');
    }));
    test('execute with script "watch-test"', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield runSnapshotTest('script-watch-test', '@dash4/plugin-dependencies @dash4/plugin-terminal @dash4/plugin-code-coverage');
    }));
    test('execute with script "watch-test-jest"', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield runSnapshotTest('script-watch-test-jest', '@dash4/plugin-dependencies @dash4/plugin-terminal @dash4/plugin-code-coverage');
    }));
    test('execute in create-react-app setup', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield runSnapshotTest('script-cra', '@dash4/plugin-dependencies @dash4/plugin-terminal @dash4/plugin-code-coverage @dash4/plugin-npm-scripts');
    }));
    test('execute with script "storybook"', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield runSnapshotTest('script-storybook', '@dash4/plugin-dependencies @dash4/plugin-terminal');
    }));
    test('execute with all possible scripts', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield runSnapshotTest('all-scripts', '@dash4/plugin-dependencies @dash4/plugin-terminal @dash4/plugin-readme @dash4/plugin-code-coverage @dash4/plugin-npm-scripts');
    }));
    test.skip('execute in lerna repo', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const testId = 'lerna';
        const cwd = path_1.default.join(tempDir, testId);
        yield make_dir_1.default(cwd);
        // add empty dir
        yield make_dir_1.default(path_1.default.join(cwd, 'packages/empty'));
        yield cpy_1.default([`src/__mocks__/${testId}/_package.json`], `__tmp__/${testId}`, {
            cwd: path_1.default.join(__dirname, '..'),
            rename: () => `package.json`,
        });
        yield cpy_1.default([`src/__mocks__/${testId}/README.md`], `__tmp__/${testId}`, {
            cwd: path_1.default.join(__dirname, '..'),
        });
        yield cpy_1.default([`src/__mocks__/${testId}/_lerna.json`], `__tmp__/${testId}`, {
            cwd: path_1.default.join(__dirname, '..'),
            rename: () => `lerna.json`,
        });
        // create configs directory
        yield make_dir_1.default(path_1.default.join(cwd, 'configs'));
        try {
            yield cpy_1.default([`*/_package.json`], `../../../../__tmp__/${testId}/configs/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/configs`),
                parents: true,
                rename: () => `package.json`,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        try {
            yield cpy_1.default([`*/README.md`], `../../../../__tmp__/${testId}/configs/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/configs`),
                parents: true,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        // create packages directory
        yield make_dir_1.default(path_1.default.join(cwd, 'packages'));
        try {
            yield cpy_1.default([`*/_package.json`], `../../../../__tmp__/${testId}/packages/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/packages`),
                parents: true,
                rename: () => `package.json`,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        try {
            yield cpy_1.default([`*/README.md`], `../../../../__tmp__/${testId}/packages/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/packages`),
                parents: true,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        yield _1.init(cwd, getOptions(cwd));
        const fileData = yield getTempFile(testId, 'dash4.config.js');
        const tree = react_test_renderer_1.default.create(react_1.default.createElement("div", { dangerouslySetInnerHTML: { __html: fileData } })).toJSON();
        expect(tree).toMatchSnapshot();
        expect(execaMock).toHaveBeenCalledWith('npm', `i -D @dash4/server @dash4/plugin-dependencies @dash4/plugin-readme @dash4/plugin-code-coverage @dash4/plugin-npm-scripts @dash4/plugin-terminal`.split(' '));
    }));
    test.skip('execute in yarn workspaces repo', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const testId = 'yarn-workspaces';
        const cwd = path_1.default.join(tempDir, testId);
        yield make_dir_1.default(cwd);
        // add empty dir
        yield make_dir_1.default(path_1.default.join(cwd, 'packages/empty'));
        yield cpy_1.default([`src/__mocks__/${testId}/_package.json`], `__tmp__/${testId}`, {
            cwd: path_1.default.join(__dirname, '..'),
            rename: () => `package.json`,
        });
        yield cpy_1.default([`src/__mocks__/${testId}/README.md`], `__tmp__/${testId}`, {
            cwd: path_1.default.join(__dirname, '..'),
        });
        yield cpy_1.default([`src/__mocks__/${testId}/_lerna.json`], `__tmp__/${testId}`, {
            cwd: path_1.default.join(__dirname, '..'),
            rename: () => `lerna.json`,
        });
        // create configs directory
        yield make_dir_1.default(path_1.default.join(cwd, 'configs'));
        try {
            yield cpy_1.default([`*/_package.json`], `../../../../__tmp__/${testId}/configs/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/configs`),
                parents: true,
                rename: () => `package.json`,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        try {
            yield cpy_1.default([`*/README.md`], `../../../../__tmp__/${testId}/configs/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/configs`),
                parents: true,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        // create packages directory
        yield make_dir_1.default(path_1.default.join(cwd, 'packages'));
        try {
            yield cpy_1.default([`*/_package.json`], `../../../../__tmp__/${testId}/packages/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/packages`),
                parents: true,
                rename: () => `package.json`,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        try {
            yield cpy_1.default([`*/README.md`], `../../../../__tmp__/${testId}/packages/`, {
                cwd: path_1.default.join(__dirname, `__mocks__/${testId}/packages`),
                parents: true,
            });
        }
        catch (err) {
            // eslint-disable-next-line
        }
        yield _1.init(cwd, getOptions(cwd));
        const fileData = yield getTempFile(testId, 'dash4.config.js');
        const tree = react_test_renderer_1.default.create(react_1.default.createElement("div", { dangerouslySetInnerHTML: { __html: fileData } })).toJSON();
        expect(tree).toMatchSnapshot();
        expect(execaMock).toHaveBeenCalledWith('yarn', `add -D -W @dash4/server @dash4/plugin-dependencies @dash4/plugin-terminal @dash4/plugin-readme @dash4/plugin-code-coverage @dash4/plugin-npm-scripts`.split(' '));
    }));
});
//# sourceMappingURL=index.spec.js.map